@@ -1,6 +1,6 @@
 /* kgx-terminal.c
  *
- * Copyright 2019 Zander Brown
+ * Copyright 2019-2024 Zander Brown
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #include "kgx-config.h"
 
 #include <glib/gi18n.h>
-#include <adwaita.h>
+
 #include <vte/vte.h>
 #define PCRE2_CODE_UNIT_WIDTH 0
 #include <pcre2.h>
@@ -38,6 +38,7 @@
 #include "kgx-terminal.h"
 #include "kgx-despatcher.h"
 #include "kgx-settings.h"
+#include "kgx-paste-dialog.h"
 #include "kgx-marshals.h"
 
 /*       Regex adapted from TerminalWidget.vala in Pantheon Terminal       */
@@ -72,8 +73,6 @@ static const char *links[KGX_TERMINAL_N_LINK_REGEX] = {
 
 /**
  * KgxTerminal:
- * @theme: the palette to use, see #KgxTerminal:theme
- * @actions: action map for the context menu
  * @current_url: the address under the cursor
  * @match_id: regex ids for finding hyperlinks
  *
@@ -84,17 +83,13 @@ struct _KgxTerminal {
 
   KgxSettings   *settings;
   GSignalGroup  *settings_signals;
-
-  GMenuModel    *context_model;
-  GtkWidget  *popup_menu;
+  GCancellable  *cancellable;
 
   KgxDespatcher *despatcher;
 
   /* Hyperlinks */
   char       *current_url;
   int         match_id[KGX_TERMINAL_N_LINK_REGEX];
-
-  gboolean    popup_is_touch;
 };
 
 
@@ -103,6 +98,7 @@ G_DEFINE_TYPE (KgxTerminal, kgx_terminal, VTE_TYPE_TERMINAL)
 enum {
   PROP_0,
   PROP_SETTINGS,
+  PROP_CANCELLABLE,
   PROP_PATH,
   LAST_PROP
 };
@@ -122,8 +118,9 @@ kgx_terminal_dispose (GObject *object)
 {
   KgxTerminal *self = KGX_TERMINAL (object);
 
+  g_clear_object (&self->cancellable);
+
   g_clear_pointer (&self->current_url, g_free);
-  g_clear_pointer (&self->popup_menu, gtk_widget_unparent);
 
   g_clear_object (&self->settings);
 
@@ -136,7 +133,6 @@ kgx_terminal_dispose (GObject *object)
 static void
 update_terminal_colours (KgxTerminal *self)
 {
-  KgxTheme current_theme;
   KgxTheme resolved_theme;
   GdkRGBA fg;
   GdkRGBA bg;
@@ -165,19 +161,7 @@ update_terminal_colours (KgxTerminal *self)
     return;
   }
 
-  g_object_get (self->settings, "theme", &current_theme, NULL);
-
-  if (current_theme == KGX_THEME_AUTO) {
-    AdwStyleManager *manager = adw_style_manager_get_default ();
-
-    if (adw_style_manager_get_dark (manager)) {
-      resolved_theme = KGX_THEME_NIGHT;
-    } else {
-      resolved_theme = KGX_THEME_DAY;
-    }
-  } else {
-    resolved_theme = current_theme;
-  }
+  resolved_theme = kgx_settings_get_resolved_theme (self->settings);
 
   switch (resolved_theme) {
     case KGX_THEME_HACKER:
@@ -210,8 +194,15 @@ kgx_terminal_set_property (GObject      *object,
 
   switch (property_id) {
     case PROP_SETTINGS:
-      g_set_object (&self->settings, g_value_get_object (value));
-      update_terminal_colours (self);
+      if (g_set_object (&self->settings, g_value_get_object (value))) {
+        update_terminal_colours (self);
+        g_object_notify_by_pspec (object, pspec);
+      }
+      break;
+    case PROP_CANCELLABLE:
+      if (g_set_object (&self->cancellable, g_value_get_object (value))) {
+        g_object_notify_by_pspec (object, pspec);
+      }
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -234,6 +225,9 @@ kgx_terminal_get_property (GObject    *object,
     case PROP_SETTINGS:
       g_value_set_object (value, self->settings);
       break;
+    case PROP_CANCELLABLE:
+      g_value_set_object (value, self->cancellable);
+      break;
     case PROP_PATH:
       if ((uri = vte_terminal_get_current_file_uri (VTE_TERMINAL (self)))) {
         path = g_file_new_for_uri (uri);
@@ -262,14 +256,14 @@ have_url_under_pointer (KgxTerminal *self,
   hyperlink = vte_terminal_check_hyperlink_at (VTE_TERMINAL (self), x, y);
 
   if (G_UNLIKELY (hyperlink)) {
-    g_set_str (&self->current_url, g_steal_pointer (&hyperlink));
+    g_set_str (&self->current_url, hyperlink);
     current = TRUE;
   } else {
     match = vte_terminal_check_match_at (VTE_TERMINAL (self), x, y, &match_id);
 
     for (int i = 0; i < KGX_TERMINAL_N_LINK_REGEX; i++) {
       if (self->match_id[i] == match_id) {
-        g_set_str (&self->current_url, g_steal_pointer (&match));
+        g_set_str (&self->current_url, match);
         current = TRUE;
         break;
       }
@@ -280,66 +274,6 @@ have_url_under_pointer (KgxTerminal *self,
 }
 
 
-static void
-update_menu_position (KgxTerminal *self)
-{
-  if (!self->popup_menu)
-    return;
-
-  gtk_popover_set_position (GTK_POPOVER (self->popup_menu),
-                            self->popup_is_touch ? GTK_POS_TOP : GTK_POS_BOTTOM);
-
-  gtk_popover_set_has_arrow (GTK_POPOVER (self->popup_menu), self->popup_is_touch);
-
-  if (self->popup_is_touch) {
-    gtk_widget_set_halign (self->popup_menu, GTK_ALIGN_FILL);
-  } else if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_RTL) {
-    gtk_widget_set_halign (self->popup_menu, GTK_ALIGN_END);
-  } else {
-    gtk_widget_set_halign (self->popup_menu, GTK_ALIGN_START);
-  }
-}
-
-
-static void
-context_menu (KgxTerminal *self,
-              double       x,
-              double       y,
-              gboolean     touch)
-{
-  gboolean value = have_url_under_pointer (self, x, y);
-
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "term.open-link", value);
-  gtk_widget_action_set_enabled (GTK_WIDGET (self), "term.copy-link", value);
-
-  if (G_UNLIKELY (!self->popup_menu)) {
-    self->popup_menu = gtk_popover_menu_new_from_model (G_MENU_MODEL (self->context_model));
-
-    gtk_widget_set_parent (self->popup_menu, GTK_WIDGET (self));
-  }
-
-  self->popup_is_touch = touch;
-
-  update_menu_position (self);
-
-  if (G_LIKELY (x > -1 && y > -1)) {
-    GdkRectangle rect = { x, y, 1, 1 };
-    gtk_popover_set_pointing_to (GTK_POPOVER (self->popup_menu), &rect);
-  } else {
-    gtk_popover_set_pointing_to (GTK_POPOVER (self->popup_menu), NULL);
-  }
-
-  gtk_popover_popup (GTK_POPOVER (self->popup_menu));
-}
-
-
-static void
-menu_popup_activated (KgxTerminal *self)
-{
-  context_menu (self, 1, 1, FALSE);
-}
-
-
 static void
 did_open (GObject *source, GAsyncResult *res, gpointer user_data)
 {
@@ -371,7 +305,7 @@ open_link (KgxTerminal *self)
   kgx_despatcher_open (self->despatcher,
                        self->current_url,
                        GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self))),
-                       NULL,
+                       self->cancellable,
                        did_open,
                        g_object_ref (self));
 }
@@ -404,17 +338,20 @@ copy_activated (KgxTerminal *self)
 
 
 static void
-got_text (GdkClipboard *cb,
+got_text (GObject      *source,
           GAsyncResult *result,
-          KgxTerminal  *self)
+          gpointer      user_data)
 {
-  g_autofree char *text = NULL;
+  g_autoptr (KgxTerminal) self = user_data;
   g_autoptr (GError) error = NULL;
+  g_autofree char *text = NULL;
 
   /* Get the resulting text of the read operation */
-  text = gdk_clipboard_read_text_finish (cb, result, &error);
+  text = gdk_clipboard_read_text_finish (GDK_CLIPBOARD (source), result, &error);
 
-  if (error) {
+  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+    return;
+  } else if (error) {
     g_critical ("Couldn't paste text: %s\n", error->message);
     return;
   }
@@ -428,7 +365,10 @@ paste_activated (KgxTerminal *self)
 {
   GdkClipboard *cb = gtk_widget_get_clipboard (GTK_WIDGET (self));
 
-  gdk_clipboard_read_text_async (cb, NULL, (GAsyncReadyCallback) got_text, self);
+  gdk_clipboard_read_text_async (cb,
+                                 self->cancellable,
+                                 got_text,
+                                 g_object_ref (self));
 }
 
 
@@ -479,7 +419,7 @@ show_in_files_activated (KgxTerminal *self)
     kgx_despatcher_show_item (self->despatcher,
                               uri,
                               parent,
-                              NULL,
+                              self->cancellable,
                               did_show_item,
                               NULL);
     return;
@@ -495,7 +435,7 @@ show_in_files_activated (KgxTerminal *self)
   kgx_despatcher_show_folder (self->despatcher,
                               uri,
                               parent,
-                              NULL,
+                              self->cancellable,
                               did_show_folder,
                               NULL);
 }
@@ -512,9 +452,6 @@ kgx_terminal_size_allocate (GtkWidget *widget,
   KgxTerminal *self = KGX_TERMINAL (widget);
   VteTerminal *term = VTE_TERMINAL (self);
 
-  if (self->popup_menu)
-    gtk_popover_present (GTK_POPOVER (self->popup_menu));
-
   GTK_WIDGET_CLASS (kgx_terminal_parent_class)->size_allocate (widget, width, height, baseline);
 
   rows = vte_terminal_get_row_count (term);
@@ -524,18 +461,6 @@ kgx_terminal_size_allocate (GtkWidget *widget,
 }
 
 
-static void
-kgx_terminal_direction_changed (GtkWidget        *widget,
-                                GtkTextDirection  previous_direction)
-{
-  KgxTerminal *self = KGX_TERMINAL (widget);
-
-  GTK_WIDGET_CLASS (kgx_terminal_parent_class)->direction_changed (widget, previous_direction);
-
-  update_menu_position (self);
-}
-
-
 static gboolean
 kgx_terminal_query_tooltip (GtkWidget  *widget,
                             int         x,
@@ -598,6 +523,26 @@ location_changed (KgxTerminal *self)
 }
 
 
+static void
+setup_context_menu (KgxTerminal *self, VteEventContext *context)
+{
+  gboolean enabled;
+
+  if (context) {
+    double x, y;
+
+    vte_event_context_get_coordinates (context, &x, &y);
+
+    enabled = have_url_under_pointer (self, x, y);
+  } else {
+    enabled = FALSE;
+  }
+
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "term.open-link", enabled);
+  gtk_widget_action_set_enabled (GTK_WIDGET (self), "term.copy-link", enabled);
+}
+
+
 static void
 pressed (GtkGestureClick *gesture,
          int              n_presses,
@@ -605,7 +550,6 @@ pressed (GtkGestureClick *gesture,
          double           y,
          KgxTerminal     *self)
 {
-  GdkEvent *event;
   GdkModifierType state;
   guint button;
 
@@ -614,15 +558,6 @@ pressed (GtkGestureClick *gesture,
     return;
   }
 
-  event = gtk_event_controller_get_current_event (GTK_EVENT_CONTROLLER (gesture));
-
-  if (gdk_event_triggers_context_menu (event)) {
-    context_menu (self, x, y, FALSE);
-    gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-
-    return;
-  }
-
   state = gtk_event_controller_get_current_event_state (GTK_EVENT_CONTROLLER (gesture));
   button = gtk_gesture_single_get_current_button (GTK_GESTURE_SINGLE (gesture));
 
@@ -640,17 +575,6 @@ pressed (GtkGestureClick *gesture,
 }
 
 
-static void
-long_pressed (GtkGestureLongPress *gesture,
-              double               x,
-              double               y,
-              KgxTerminal         *self)
-{
-  context_menu (self, x, y, TRUE);
-  gtk_gesture_set_state (GTK_GESTURE (gesture), GTK_EVENT_SEQUENCE_CLAIMED);
-}
-
-
 static gboolean
 scroll (GtkEventControllerScroll *scroll,
         double                   dx,
@@ -681,7 +605,6 @@ kgx_terminal_class_init (KgxTerminalClass *klass)
   object_class->get_property = kgx_terminal_get_property;
 
   widget_class->size_allocate = kgx_terminal_size_allocate;
-  widget_class->direction_changed = kgx_terminal_direction_changed;
   widget_class->query_tooltip = kgx_terminal_query_tooltip;
 
   term_class->selection_changed = kgx_terminal_selection_changed;
@@ -691,7 +614,12 @@ kgx_terminal_class_init (KgxTerminalClass *klass)
   pspecs[PROP_SETTINGS] =
     g_param_spec_object ("settings", NULL, NULL,
                          KGX_TYPE_SETTINGS,
-                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
+  pspecs[PROP_CANCELLABLE] =
+    g_param_spec_object ("cancellable", NULL, NULL,
+                         G_TYPE_CANCELLABLE,
+                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   pspecs[PROP_PATH] =
     g_param_spec_object ("path", NULL, NULL,
@@ -728,18 +656,13 @@ kgx_terminal_class_init (KgxTerminalClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                KGX_APPLICATION_PATH "kgx-terminal.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, KgxTerminal, context_model);
   gtk_widget_class_bind_template_child (widget_class, KgxTerminal, settings_signals);
 
   gtk_widget_class_bind_template_callback (widget_class, location_changed);
+  gtk_widget_class_bind_template_callback (widget_class, setup_context_menu);
   gtk_widget_class_bind_template_callback (widget_class, pressed);
-  gtk_widget_class_bind_template_callback (widget_class, long_pressed);
   gtk_widget_class_bind_template_callback (widget_class, scroll);
 
-  gtk_widget_class_install_action (widget_class,
-                                   "menu.popup",
-                                   NULL,
-                                   (GtkWidgetActionActivateFunc) menu_popup_activated);
   gtk_widget_class_install_action (widget_class,
                                    "term.open-link",
                                    NULL,
@@ -764,56 +687,6 @@ kgx_terminal_class_init (KgxTerminalClass *klass)
                                    "term.show-in-files",
                                    NULL,
                                    (GtkWidgetActionActivateFunc) show_in_files_activated);
-
-  gtk_widget_class_add_binding_action (widget_class,
-                                       GDK_KEY_F10, GDK_SHIFT_MASK,
-                                       "menu.popup",
-                                       NULL);
-  gtk_widget_class_add_binding_action (widget_class,
-                                       GDK_KEY_Menu, 0,
-                                       "menu.popup",
-                                       NULL);
-}
-
-
-typedef struct {
-  KgxTerminal *dest;
-  char        *text;
-} PasteData;
-
-
-static void
-clear_paste_data (gpointer data)
-{
-  PasteData *self = data;
-
-  g_clear_weak_pointer (&self->dest);
-  g_free (self->text);
-  g_free (self);
-}
-
-
-G_DEFINE_AUTOPTR_CLEANUP_FUNC (PasteData, clear_paste_data)
-
-static void
-paste_response (PasteData *data)
-{
-  g_autoptr (PasteData) paste = data;
-
-  vte_terminal_paste_text (VTE_TERMINAL (paste->dest), paste->text);
-}
-
-
-static void
-dark_changed (KgxTerminal *self)
-{
-  KgxTheme theme;
-
-  g_object_get (self->settings, "theme", &theme, NULL);
-
-  if (theme == KGX_THEME_AUTO) {
-    update_terminal_colours (self);
-  }
 }
 
 
@@ -850,57 +723,67 @@ kgx_terminal_init (KgxTerminal *self)
                                         "pointer");
   }
 
-  g_signal_connect_object (adw_style_manager_get_default (),
-                           "notify::dark", G_CALLBACK (dark_changed),
-                           self, G_CONNECT_SWAPPED);
-
   g_signal_group_connect_swapped (self->settings_signals,
-                                  "notify::theme", G_CALLBACK (update_terminal_colours),
+                                  "notify::resolved-theme", G_CALLBACK (update_terminal_colours),
                                   self);
 }
 
 
+static void
+got_paste (GObject      *source,
+           GAsyncResult *res,
+           gpointer      user_data)
+{
+  g_autoptr (KgxPasteDialog) dialogue = KGX_PASTE_DIALOG (source);
+  g_autoptr (KgxTerminal) self = user_data;
+  g_autoptr (GError) error = NULL;
+  KgxPasteDialogResult result;
+  const char *content;
+
+  result = kgx_paste_dialog_run_finish (dialogue, res, &content, &error);
+
+  if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
+    result = KGX_PASTE_CANCELLED;
+  } else if (error) {
+    g_critical ("terminal [paste]: Unexpected: %s", error->message);
+    return;
+  }
+
+  if (result == KGX_PASTE_CANCELLED) {
+    return;
+  }
+
+  vte_terminal_paste_text (VTE_TERMINAL (self), content);
+}
+
+
 void
 kgx_terminal_accept_paste (KgxTerminal *self,
                            const char  *text)
 {
   g_autofree char *striped = g_strchug (g_strdup (text));
-  g_autoptr (PasteData) paste = g_new0 (PasteData, 1);
-  gsize len;
+  size_t len;
 
-  if (!text || !text[0]) {
+  g_return_if_fail (KGX_IS_TERMINAL (self));
+
+  if (G_UNLIKELY (!text || !text[0])) {
     return;
   }
 
-  len = strlen (text);
-
-  g_set_weak_pointer (&paste->dest, self);
-  g_set_str (&paste->text, text);
+  len = strlen (striped);
 
   if (g_strstr_len (striped, len, "sudo") != NULL &&
       g_strstr_len (striped, len, "\n") != NULL) {
-    GtkWidget *dlg = adw_message_dialog_new (GTK_WINDOW (gtk_widget_get_root (GTK_WIDGET (self))),
-                                             _("You are pasting a command that runs as an administrator"),
-                                             NULL);
-    adw_message_dialog_format_body (ADW_MESSAGE_DIALOG (dlg),
-                                    // TRANSLATORS: %s is the command being pasted
-                                    _("Make sure you know what the command does:\n%s"),
-                                    text);
-    adw_message_dialog_add_responses (ADW_MESSAGE_DIALOG (dlg),
-                                      "cancel", _("_Cancel"),
-                                      "paste", _("_Paste"),
-                                      NULL);
-    adw_message_dialog_set_response_appearance (ADW_MESSAGE_DIALOG (dlg),
-                                                "paste",
-                                                ADW_RESPONSE_DESTRUCTIVE);
+    KgxPasteDialog *paste = g_object_new (KGX_TYPE_PASTE_DIALOG,
+                                          "content", text,
+                                          "transient-for", gtk_widget_get_root (GTK_WIDGET (self)),
+                                          NULL);
 
-    g_signal_connect_swapped (dlg,
-                              "response::paste",
-                              G_CALLBACK (paste_response),
-                              g_steal_pointer (&paste));
-
-    gtk_window_present (GTK_WINDOW (dlg));
+    kgx_paste_dialog_run (paste,
+                          self->cancellable,
+                          got_paste,
+                          g_object_ref (self));
   } else {
-    paste_response (g_steal_pointer (&paste));
+    vte_terminal_paste_text (VTE_TERMINAL (self), text);
   }
 }
